@@ -1,31 +1,10 @@
-# Copyright (c) 2012-2014 roger
-# Copyright (c) 2012-2015 Tycho Andersen
-# Copyright (c) 2013 dequis
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2013 Craig Barnes
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2018 Piotr Przymus
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+import os
+from pathlib import Path
+
 import cairocffi
 
 try:
+    from xdg.DesktopEntry import DesktopEntry
     from xdg.IconTheme import getIconPath
 
     has_xdg = True
@@ -38,6 +17,14 @@ from libqtile.images import Img
 from libqtile.log_utils import logger
 from libqtile.widget import base
 
+data_home = os.environ.get("XDG_DATA_HOME", "~/.local/share")
+
+DESKTOP_LOCATIONS = [
+    Path(data_home) / "applications",
+    Path("/usr/local/share/applications"),
+    Path("/usr/share/applications"),
+]
+
 
 class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
     """Displays the icon and name of each window in the current group
@@ -49,7 +36,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
     to use theme icons and to display icons on Wayland.
     """
 
-    orientations = base.ORIENTATION_HORIZONTAL
+    orientations = base.ORIENTATION_BOTH
     defaults = [
         ("font", "sans", "Default font"),
         ("fontsize", None, "Font size. Calculated if None."),
@@ -99,7 +86,12 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             "   return text"
             "then set option parse_text=my_func",
         ),
-        ("spacing", None, "Spacing between tasks.(if set to None, will be equal to margin_x)"),
+        (
+            "spacing",
+            None,
+            "Spacing between tasks. If set to None, defaults to margin_x in "
+            "horizontal bars and margin_y in vertical bars.",
+        ),
         (
             "txt_minimized",
             "_ ",
@@ -188,26 +180,22 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
     def __init__(self, **config):
         base._Widget.__init__(self, libqtile.bar.STRETCH, **config)
         self.add_defaults(TaskList.defaults)
-        self.add_defaults(base.PaddingMixin.defaults)
-        self.add_defaults(base.MarginMixin.defaults)
         self._icons_cache = {}
         self._box_end_positions = []
         self.markup = False
         self.clicked = None
-        if self.spacing is None:
-            self.spacing = self.margin_x
 
         self.add_callbacks({"Button1": self.select_window})
 
     def box_width(self, text):
         """
-        calculate box width for given text.
+        Calculate box width for given text.
         If max_title_width is given, the returned width is limited to it.
         """
-        width, _ = self.drawer.max_layout_size(
+        width, height = self.drawer.max_layout_size(
             [text], self.font, self.fontsize, markup=self.markup
         )
-        width = width + 2 * (self.padding_x + self.borderwidth)
+        width = width + 2 * (self.padding_side + self.borderwidth)
         return width
 
     def get_taskname(self, window):
@@ -219,20 +207,6 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         state = ""
         markup_str = self.markup_normal
 
-        # Enforce markup and new string format behaviour when
-        # at least one markup_* option is used.
-        # Mixing non markup and markup may cause problems.
-        if (
-            self.markup_minimized
-            or self.markup_maximized
-            or self.markup_floating
-            or self.markup_focused
-            or self.markup_focused_floating
-        ):
-            enforce_markup = True
-        else:
-            enforce_markup = False
-
         if window is None:
             pass
         elif window.minimized:
@@ -265,11 +239,10 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
                 logger.exception("parse_text function failed:")
 
         # Emulate default widget behavior if markup_str is None
-        if enforce_markup and markup_str is None:
+        if self.markup and markup_str is None:
             markup_str = f"{state}{{}}"
 
         if markup_str is not None:
-            self.markup = True
             window_name = pangocffi.markup_escape_text(window_name)
             return markup_str.format(window_name)
 
@@ -277,21 +250,24 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
 
     @property
     def windows(self):
+        windows = []
         if self.qtile.core.name == "x11":
-            windows = []
             for w in self.bar.screen.group.windows:
                 wm_states = list(w.window.get_property("_NET_WM_STATE", "ATOM", unpack=int))
                 skip_taskbar = w.qtile.core.conn.atoms["_NET_WM_STATE_SKIP_TASKBAR"]
                 if w.window.get_wm_type() in ("normal", None) and skip_taskbar not in wm_states:
                     windows.append(w)
-            return windows
-        return self.bar.screen.group.windows
+        else:
+            for w in self.bar.screen.group.windows:
+                if w.get_wm_type() in ("normal", None) and not w.skip_taskbar:
+                    windows.append(w)
+
+        return windows
 
     @property
     def max_width(self):
-        width = self.bar.width
-        width -= sum(
-            w.width
+        width = self.bar.length - sum(
+            w.length
             for w in self.bar.widgets
             if w is not self and w.length_type != libqtile.bar.STRETCH
         )
@@ -311,7 +287,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             return []
 
         # Determine available and max average width for task name boxes.
-        width_total = self.max_width - 2 * self.margin_x - (window_count - 1) * self.spacing
+        width_total = self.max_width - 2 * self.margin_side - (window_count - 1) * self.spacing
         width_avg = width_total / window_count
 
         names = [self.get_taskname(w) for w in windows]
@@ -332,7 +308,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             width_boxes = [
                 (
                     self.box_width(names[idx])
-                    + ((self.icon_size + self.padding_x) if icons[idx] else 0)
+                    + ((self.icon_size + self.padding_side) if icons[idx] else 0)
                 )
                 for idx in range(window_count)
             ]
@@ -367,6 +343,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
     def _configure(self, qtile, bar):
         base._Widget._configure(self, qtile, bar)
 
+        if self.spacing is None:
+            self.spacing = self.margin_side
+
         if not self.stretch:
             self.length_type = libqtile.bar.CALCULATED
 
@@ -385,13 +364,33 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             self.icon_size = 0
 
         if self.icon_size is None:
-            self.icon_size = self.bar.height - 2 * (self.borderwidth + self.margin_y)
+            self.icon_size = self.bar.size - 2 * (self.borderwidth + self.margin_top)
 
         if self.fontsize is None:
-            calc = self.bar.height - self.margin_y * 2 - self.borderwidth * 2 - self.padding_y * 2
+            calc = (
+                self.bar.size - self.margin_top * 2 - self.borderwidth * 2 - self.padding_top * 2
+            )
             self.fontsize = max(calc, 1)
+
+        # Enforce markup and new string format behaviour when
+        # at least one markup_* option is used.
+        # Mixing non markup and markup may cause problems.
+        self.markup = bool(
+            self.markup_normal
+            or self.markup_minimized
+            or self.markup_maximized
+            or self.markup_floating
+            or self.markup_focused
+            or self.markup_focused_floating
+        )
         self.layout = self.drawer.textlayout(
-            "", "ffffff", self.font, self.fontsize, self.fontshadow, wrap=False
+            "",
+            "ffffff",
+            self.font,
+            self.fontsize,
+            self.fontshadow,
+            wrap=False,
+            markup=self.markup,
         )
         self.setup_hooks()
 
@@ -418,13 +417,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         hook.subscribe.client_killed(self.remove_icon_cache)
 
     def drawtext(self, text, textcolor, width):
-        if self.markup:
-            self.layout.markup = self.markup
-
         self.layout.text = text
-
-        self.layout.font_family = self.font
-        self.layout.font_size = self.fontsize
         self.layout.colour = textcolor
         if width is not None:
             self.layout.width = width
@@ -442,8 +435,8 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
     ):
         self.drawtext(text, textcolor, width)
 
-        icon_padding = (self.icon_size + self.padding_x) if icon else 0
-        padding_x = [self.padding_x + icon_padding, self.padding_x]
+        icon_padding = (self.icon_size + self.padding_side) if icon else 0
+        pad_x = [self.padding_side + icon_padding, self.padding_side]
 
         if bordercolor is None:
             # border colour is set to None when we don't want to draw a border at all
@@ -455,19 +448,25 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             border_width = self.borderwidth
             framecolor = bordercolor
 
-        framed = self.layout.framed(border_width, framecolor, padding_x, self.padding_y)
+        framed = self.layout.framed(border_width, framecolor, pad_x, self.padding_top)
         if block and bordercolor is not None:
-            framed.draw_fill(offset, self.margin_y, rounded)
+            framed.draw_fill(offset, self.margin_top, rounded)
         else:
-            framed.draw(offset, self.margin_y, rounded)
+            framed.draw(offset, self.margin_top, rounded)
 
         if icon:
             self.draw_icon(icon, offset)
 
     def get_clicked(self, x, y):
-        box_start = self.margin_x
+        box_start = self.margin_side
+        if self.bar.horizontal:
+            pos = x
+        elif self.bar.screen.left is self.bar:
+            pos = self.length - y
+        else:
+            pos = y
         for box_end, win in zip(self._box_end_positions, self.windows):
-            if box_start <= x <= box_end:
+            if box_start <= pos <= box_end:
                 return win
             else:
                 box_start = box_end + self.spacing
@@ -506,6 +505,33 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
 
         return img
 
+    def _read_desktop_file(self, desktopfile):
+        icon = DesktopEntry(desktopfile).getIcon()
+        if icon and Path(icon).expanduser().exists():
+            return icon
+
+    def _find_desktop_entry(self, classes):
+        for p in DESKTOP_LOCATIONS:
+            for cl in classes:
+                for app in set([cl, cl.lower()]):
+                    f = (p / f"{app}.desktop").expanduser()
+                    if f.exists():
+                        icon = self._read_desktop_file(f)
+                        if icon:
+                            return icon
+
+    def _get_desktop_icon(self, window):
+        classes = window.get_wm_class()
+
+        if not classes:
+            return None
+
+        icon = self._find_desktop_entry(classes)
+
+        if icon:
+            img = Img.from_path(icon)
+            return img.surface
+
     def _get_theme_icon(self, window):
         classes = window.get_wm_class()
 
@@ -544,6 +570,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         if self.qtile.core.name == "x11":
             img = self._get_class_icon(window)
 
+        if img is None and has_xdg and self.theme_mode != "preferred":
+            img = self._get_desktop_icon(window)
+
         if self.theme_mode == "preferred" or (self.theme_mode == "fallback" and img is None):
             xdg_img = self._get_theme_icon(window)
             if xdg_img:
@@ -568,8 +597,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
         if not surface:
             return
 
-        x = offset + self.borderwidth + self.padding_x
-        y = (self.height - self.icon_size) // 2
+        x = offset + self.borderwidth + self.padding_side
+        size = self.height if self.bar.horizontal else self.width
+        y = (size - self.icon_size) // 2
 
         self.drawer.ctx.save()
         self.drawer.ctx.translate(x, y)
@@ -579,7 +609,9 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
 
     def draw(self):
         self.drawer.clear(self.background or self.bar.background)
-        offset = self.margin_x
+        offset = self.margin_side
+        self.drawer.ctx.save()
+        self.rotate_drawer()
 
         self._box_end_positions = []
         for w, icon, task, bw in self.calc_box_widths():
@@ -601,7 +633,7 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
                 text_color = self.foreground
 
             textwidth = (
-                bw - 2 * self.padding_x - ((self.icon_size + self.padding_x) if icon else 0)
+                bw - 2 * self.padding_side - ((self.icon_size + self.padding_side) if icon else 0)
             )
             self.drawbox(
                 offset,
@@ -615,4 +647,5 @@ class TaskList(base._Widget, base.PaddingMixin, base.MarginMixin):
             )
             offset += bw + self.spacing
 
-        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
+        self.drawer.ctx.restore()
+        self.draw_at_default_position()
